/**
 * name: @tresjs/post-processing
 * version: v0.4.0
 * (c) 2023
 * description: Post-processing library for TresJS
 * author: Alvaro Saburido <hola@alvarosaburido.dev> (https://github.com/alvarosabu/)
 */
import { defineComponent as _, inject as T, shallowRef as g, watchEffect as h, onUnmounted as w, computed as j, watch as F, provide as H, renderSlot as I } from "vue";
import { useTresContext as P, normalizeColor as z, useRenderLoop as X } from "@tresjs/core";
import { BloomEffect as O, EffectPass as M, GlitchEffect as k, GlitchMode as Y, OutlineEffect as A, EffectComposer as R, RenderPass as K, NormalPass as U, DepthDownsamplingPass as q } from "postprocessing";
import { HalfFloatType as G } from "three";
const x = Symbol(), ee = /* @__PURE__ */ _({
  __name: "Bloom",
  props: {
    blendFunction: {},
    intensity: {},
    kernelSize: {},
    luminanceThreshold: {},
    luminanceSmoothing: {},
    mipmapBlur: { type: Boolean }
  },
  setup(f, { expose: o }) {
    const l = f, n = T(x), t = g(null), e = g(null);
    o({ pass: t, effect: e });
    const { camera: i } = P();
    let s;
    return s = h(() => {
      !i.value || !(n != null && n.value) || (s == null || s(), !e.value && (e.value = new O(l), t.value = new M(i.value, e.value), n.value.addPass(t.value)));
    }), h(() => {
      if (!e.value)
        return;
      const a = new O();
      e.value.intensity = l.intensity !== void 0 ? l.intensity : a.intensity, e.value.kernelSize = l.kernelSize !== void 0 ? l.kernelSize : a.kernelSize, e.value.luminanceMaterial.smoothing = l.luminanceSmoothing !== void 0 ? l.luminanceSmoothing : a.luminanceMaterial.smoothing, e.value.luminanceMaterial.threshold = l.luminanceThreshold !== void 0 ? l.luminanceThreshold : a.luminanceMaterial.threshold, a.dispose();
    }), w(() => {
      var a, d, v;
      t.value && ((a = n == null ? void 0 : n.value) == null || a.removePass(t.value)), (d = e.value) == null || d.dispose(), (v = t.value) == null || v.dispose();
    }), (a, d) => null;
  }
}), le = /* @__PURE__ */ _({
  __name: "Glitch",
  props: {
    blendFunction: {},
    delay: {},
    duration: {},
    strength: {},
    mode: {},
    active: { type: Boolean },
    ratio: {},
    columns: {},
    chromaticAberrationOffset: {},
    perturbationMap: {},
    dtSize: {}
  },
  setup(f, { expose: o }) {
    const l = f, n = T(x), t = g(null), e = g(null);
    o({ pass: t, effect: e });
    const { camera: i } = P();
    let s;
    return s = h(() => {
      !i.value || !(n != null && n.value) || (s == null || s(), !e.value && (e.value = new k(l), t.value = new M(i.value, e.value), n.value.addPass(t.value)));
    }), h(() => {
      if (!e.value)
        return;
      const a = new k(), d = () => l.mode !== void 0 ? l.active === !1 ? Y.DISABLED : l.mode : a.mode;
      e.value.mode = d(), e.value.ratio = l.ratio !== void 0 ? l.ratio : a.ratio, e.value.delay = l.delay !== void 0 ? l.delay : a.delay, e.value.columns = l.columns !== void 0 ? l.columns : a.columns, e.value.duration = l.duration !== void 0 ? l.duration : a.duration, e.value.strength = l.strength !== void 0 ? l.strength : a.strength, e.value.perturbationMap = l.perturbationMap !== void 0 ? l.perturbationMap : a.perturbationMap, e.value.chromaticAberrationOffset = l.chromaticAberrationOffset !== void 0 ? l.chromaticAberrationOffset : a.chromaticAberrationOffset;
    }), w(() => {
      var a, d, v;
      t.value && ((a = n == null ? void 0 : n.value) == null || a.removePass(t.value)), (d = e.value) == null || d.dispose(), (v = t.value) == null || v.dispose();
    }), (a, d) => null;
  }
}), ae = /* @__PURE__ */ _({
  __name: "Outline",
  props: {
    outlinedObjects: {},
    blur: { type: Boolean },
    xRay: { type: Boolean },
    kernelSize: {},
    pulseSpeed: {},
    resolutionX: {},
    resolutionY: {},
    edgeStrength: {},
    patternScale: {},
    multisampling: {},
    blendFunction: {},
    patternTexture: {},
    resolutionScale: {},
    hiddenEdgeColor: {},
    visibleEdgeColor: {}
  },
  setup(f, { expose: o }) {
    const l = f, n = T(x), t = g(null), e = g(null);
    o({ pass: t, effect: e });
    const i = (u) => u !== void 0 ? z(u).getHex() : void 0, s = j(() => {
      const {
        blur: u,
        xRay: c,
        kernelSize: p,
        pulseSpeed: r,
        resolutionX: m,
        resolutionY: S,
        patternScale: B,
        edgeStrength: C,
        multisampling: y,
        blendFunction: E,
        patternTexture: N,
        resolutionScale: D,
        hiddenEdgeColor: $,
        visibleEdgeColor: W
      } = l;
      return {
        blur: u,
        xRay: c,
        kernelSize: p,
        pulseSpeed: r,
        resolutionX: m,
        resolutionY: S,
        patternScale: B,
        edgeStrength: C,
        blendFunction: E,
        multisampling: y,
        patternTexture: N,
        resolutionScale: D,
        hiddenEdgeColor: i($),
        visibleEdgeColor: i(W)
      };
    }), { camera: a, scene: d } = P();
    let v;
    return v = h(() => {
      !a.value || !(n != null && n.value) || !d.value || (v == null || v(), !e.value && (e.value = new A(d.value, a.value, s.value), t.value = new M(a.value, e.value), n.value.addPass(t.value)));
    }), F(
      [() => l.outlinedObjects, e],
      // watchEffect is intentionally not used here as it would result in an endless loop
      () => {
        var u;
        (u = e.value) == null || u.selection.set(l.outlinedObjects || []);
      },
      {
        immediate: !0
      }
    ), h(() => {
      if (!e.value)
        return;
      const u = new A(), {
        blur: c,
        xRay: p,
        kernelSize: r,
        pulseSpeed: m,
        patternScale: S,
        edgeStrength: B,
        multisampling: C,
        hiddenEdgeColor: y,
        visibleEdgeColor: E
      } = s.value;
      e.value.blur = c !== void 0 ? c : u.blur, e.value.xRay = p !== void 0 ? p : u.xRay, e.value.pulseSpeed = m !== void 0 ? m : u.pulseSpeed, e.value.kernelSize = r !== void 0 ? r : u.kernelSize, e.value.edgeStrength = B !== void 0 ? B : u.edgeStrength, e.value.patternScale = S !== void 0 ? S : u.patternScale, e.value.multisampling = C !== void 0 ? C : u.multisampling, e.value.hiddenEdgeColor = y !== void 0 ? z(y) : u.hiddenEdgeColor, e.value.visibleEdgeColor = E !== void 0 ? z(E) : u.visibleEdgeColor, u.dispose();
    }), w(() => {
      var u, c, p, r;
      (u = e.value) == null || u.selection.clear(), t.value && ((c = n == null ? void 0 : n.value) == null || c.removePass(t.value)), (p = e.value) == null || p.dispose(), (r = t.value) == null || r.dispose();
    }), (u, c) => null;
  }
});
let b;
function L() {
  var f;
  if (b !== void 0)
    return b;
  try {
    let o;
    const l = document.createElement("canvas");
    return b = !!(window.WebGL2RenderingContext && (o = l.getContext("webgl2"))), o && ((f = o.getExtension("WEBGL_lose_context")) == null || f.loseContext()), b;
  } catch {
    return b = !1;
  }
}
const te = /* @__PURE__ */ _({
  __name: "EffectComposer",
  props: {
    enabled: { type: Boolean, default: !0 },
    children: {},
    depthBuffer: { type: Boolean, default: void 0 },
    disableNormalPass: { type: Boolean, default: !1 },
    stencilBuffer: { type: Boolean, default: void 0 },
    resolutionScale: {},
    autoClear: { type: Boolean, default: !0 },
    multisampling: { default: 0 },
    frameBufferType: { default: G }
  },
  setup(f) {
    const o = f, { scene: l, camera: n, renderer: t, sizes: e } = P(), i = g(null);
    let s = null, a = null;
    H(x, i);
    const d = () => {
      i.value && (a = new U(l.value, n.value), a.enabled = !1, i.value.addPass(a), o.resolutionScale !== void 0 && L() && (s = new q({
        normalBuffer: a.texture,
        resolutionScale: o.resolutionScale
      }), s.enabled = !1, i.value.addPass(s)));
    };
    h(() => {
      i.value && e.width.value && e.height.value && i.value.setSize(e.width.value, e.height.value);
    });
    const v = j(() => {
      const r = new R(), m = {
        depthBuffer: o.depthBuffer !== void 0 ? o.depthBuffer : r.inputBuffer.depthBuffer,
        stencilBuffer: o.stencilBuffer !== void 0 ? o.stencilBuffer : r.inputBuffer.stencilBuffer,
        multisampling: L() ? 0 : o.multisampling !== void 0 ? o.multisampling : r.multisampling,
        frameBufferType: o.frameBufferType !== void 0 ? o.frameBufferType : G
      };
      return r.dispose(), m;
    }), u = () => {
      !t.value && !l.value && !n.value || (i.value = new R(t.value, v.value), i.value.addPass(new K(l.value, n.value)), o.disableNormalPass || d());
    }, c = F([e.height, e.width], () => {
      !e.height.value && !e.width.value || (h(u), c == null || c());
    }), { onLoop: p } = X();
    return p(({ delta: r }) => {
      if (o.enabled && t.value && i.value && e.width.value && e.height.value) {
        const m = t.value.autoClear;
        t.value.autoClear = o.autoClear, o.stencilBuffer && !o.autoClear && t.value.clearStencil(), i.value.render(r), t.value.autoClear = m;
      }
    }), w(() => {
      var r;
      (r = i.value) == null || r.dispose();
    }), (r, m) => I(r.$slots, "default");
  }
});
export {
  ee as Bloom,
  te as EffectComposer,
  le as Glitch,
  ae as Outline
};
